﻿using System;
using System.Collections.Generic;
using System.Drawing.Imaging;
using System.Globalization;
using System.Linq;
using System.Windows.Forms;
using Wunnell.Extensions;

namespace TestBed.CS
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            dateTimeIsWeekDayCheckBox.Checked = dateTimeIsWeekDayDatePicker.Value.IsWeekDay();

            enumerableListBox.DataSource = new[] {"One", "Two", "Three", "Four", "Five", "Six", "Seven", "Eight", "Nine", "Ten"};

            InitRandomPage();
            InitStringPage();
        }

        #region Data

        #endregion Data

        #region DateTime

        private void isWeekDayDatePicker_ValueChanged(object sender, EventArgs e)
        {
            dateTimeIsWeekDayCheckBox.Checked = dateTimeIsWeekDayDatePicker.Value.IsWeekDay();
        }

        private void initialDatePicker_ValueChanged(object sender, EventArgs e)
        {
            UpdateFinalDate();
        }

        private void addWeekDaysSpinners_ValueChanged(object sender, EventArgs e)
        {
            UpdateFinalDate();
        }

        private void UpdateFinalDate()
        {
            var wholeDays = dateTimeAddWeekDaysDaysSpinner.Value;
            var partialDays = ((dateTimeAddWeekDaysSecondsSpinner.Value / 60M + dateTimeAddWeekDaysMinutesSpinner.Value) / 60M + dateTimeAddWeekDaysHoursSpinner.Value) / 24M;
            var totalDays = wholeDays < 0M
                                ? wholeDays - partialDays
                                : wholeDays + partialDays;

            dateTimeAddWeekDaysTotalDaysValueLabel.Text = totalDays.ToString("n5");
            dateTimeAddWeekDaysFinalDatePicker.Value = dateTimeAddWeekDaysInitialDatePicker.Value.AddWeekDays(Convert.ToDouble(totalDays));
        }

        #endregion DateTime

        #region Enumerable

        private void enumerableRandmizeButton_Click(object sender, EventArgs e)
        {
            enumerableListBox.DataSource = enumerableListBox.Items
                                                            .Cast<string>()
                                                            .Randomize()
                                                            .ToArray();
        }

        #endregion Enumerable

        #region Image

        private void imageLoadButton_Click(object sender, EventArgs e)
        {
            using (var ofd = new OpenFileDialog())
            {
                if (ofd.ShowDialog() == DialogResult.OK)
                {
                    imageSourcePictureBox.Image?.Dispose();
                    imageSourcePictureBox.ImageLocation = ofd.FileName;
                }
            }
        }

        private void imageCopyButton_Click(object sender, EventArgs e)
        {
            imageDestinationPictureBox.Image?.Dispose();
            imageDestinationPictureBox.Image = imageSourcePictureBox.Image.ToBytes().ToImage();
        }

        private void imageCopyBitmapButton_Click(object sender, EventArgs e)
        {
            imageDestinationPictureBox.Image?.Dispose();
            imageDestinationPictureBox.Image = imageSourcePictureBox.Image.ToBytes(ImageFormat.Bmp).ToImage();
        }

        private void imageCopyJpegButton_Click(object sender, EventArgs e)
        {
            imageDestinationPictureBox.Image?.Dispose();
            imageDestinationPictureBox.Image = imageSourcePictureBox.Image.ToBytes(ImageFormat.Jpeg).ToImage();
        }

        #endregion Image

        #region Random

        private bool randomIgnoreCellValueChanged;

        private void InitRandomPage()
        {
            randomIgnoreCellValueChanged = true;

            randomMinValueCheckBox.Checked = true;
            randomMinValueSpinner.Value = 1M;
            randomMaxValueSpinner.Value = 10M;

            var totalWeighting = 10.Triangular();

            for (var i = 1; i <= 10; i++)
            {
                randomDataGridView.Rows.Add(i,
                                            i,
                                            i.AsPercentageOf(totalWeighting));
            }

            randomIgnoreCellValueChanged = false;
        }

        private void RefreshRandomWeightings()
        {
            randomIgnoreCellValueChanged = true;

            var minValue = Convert.ToInt32(randomMinValueSpinner.Value);
            var maxValue = Convert.ToInt32(randomMaxValueSpinner.Value);
            var weightingsByValue = new Dictionary<int, int?>();
            var defaultWeighting = Convert.ToInt32(randomDefaultWeightingSpinner.Value);

            for (var i = minValue; i <= maxValue; i++)
            {
                weightingsByValue.Add(i, defaultWeighting);
            }

            foreach (DataGridViewRow row in randomDataGridView.Rows)
            {
                var value = (int) row.Cells[ValueColumn.Index].Value;
                var weighting = row.Cells[WeightingColumn.Index].Value as int?;

                if (weightingsByValue.ContainsKey(value))
                {
                    weightingsByValue[value] = weighting;
                }
            }

            randomDataGridView.Rows.Clear();

            var totalWeighting = weightingsByValue.Values.Sum(w => w ?? defaultWeighting);

            foreach (var value in weightingsByValue.Keys)
            {
                var weighting = weightingsByValue[value];

                randomDataGridView.Rows.Add(value, weighting, (weighting ?? defaultWeighting).AsPercentageOf(totalWeighting));
            }

            randomIgnoreCellValueChanged = false;
        }

        private int GetWeightingForRow(DataGridViewRow row)
        {
            var cellWeighting = row.Cells[WeightingColumn.Index].Value;

            if (cellWeighting == null || cellWeighting.ToString() == string.Empty)
            {
                return Convert.ToInt32(randomDefaultWeightingSpinner.Value);
            }

            return Convert.ToInt32(cellWeighting);
        }

        private int GetRandomNumber(Random rng,
                                    bool useMinValue,
                                    int minValue,
                                    int maxValue,
                                    bool allWeightingsProvided,
                                    IDictionary<int, int> partialWeightings,
                                    int[] completeWeightings,
                                    bool useDefaultWeighting,
                                    int defaultWeighting)
        {
            int randomNumber;

            if (allWeightingsProvided)
            {
                if (useMinValue)
                {
                    randomNumber = rng.NextWithWeighting(minValue, maxValue, completeWeightings);
                }
                else
                {
                    randomNumber = rng.NextWithWeighting(maxValue, completeWeightings);
                }
            }
            else if (useDefaultWeighting)
            {
                if (useMinValue)
                {
                    randomNumber = rng.NextWithWeighting(minValue, maxValue, partialWeightings, defaultWeighting);
                }
                else
                {
                    randomNumber = rng.NextWithWeighting(maxValue, partialWeightings, defaultWeighting);
                }
            }
            else
            {
                if (useMinValue)
                {
                    randomNumber = rng.NextWithWeighting(minValue, maxValue, partialWeightings);
                }
                else
                {
                    randomNumber = rng.NextWithWeighting(maxValue, partialWeightings);
                }
            }

            return randomNumber;
        }

        private void randomMinValueCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            if (randomMinValueCheckBox.Checked)
            {
                if (randomMinValueSpinner.Value > randomMaxValueSpinner.Value)
                {
                    randomMinValueSpinner.Value = randomMaxValueSpinner.Value;
                }

                randomMaxValueSpinner.Minimum = randomMinValueSpinner.Value;
                randomMinValueSpinner.Enabled = true;
            }
            else
            {
                randomMinValueSpinner.Enabled = false;
                randomMinValueSpinner.Value = 0M;
            }
        }

        private void randomMinValueSpinner_ValueChanged(object sender, EventArgs e)
        {
            randomMaxValueSpinner.Minimum = randomMinValueSpinner.Value;
        }

        private void randomDefaultWeightingCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            randomDefaultWeightingSpinner.Enabled = randomDefaultWeightingCheckBox.Checked;

            if (!randomDefaultWeightingSpinner.Enabled)
            {
                randomDefaultWeightingSpinner.Value = 1M;
            }
        }

        private void randomRefreshButton_Click(object sender, EventArgs e)
        {
            RefreshRandomWeightings();
        }

        private void randomGenerateButton_Click(object sender, EventArgs e)
        {
            var occurrencesByValue = randomDataGridView.Rows
                                                       .Cast<DataGridViewRow>()
                                                       .ToDictionary(row => (int) row.Cells[ValueColumn.Index].Value, row => 0);
            var weightings = randomDataGridView.Rows
                                               .Cast<DataGridViewRow>()
                                               .ToDictionary(row => (int) row.Cells[ValueColumn.Index].Value,
                                                             row => row.Cells[WeightingColumn.Index].Value as int?);
            var useMinValue = randomMinValueCheckBox.Checked;
            var minValue = Convert.ToInt32(randomMinValueSpinner.Value);

            // Upper limit is exclusive.
            var maxValue = Convert.ToInt32(randomMaxValueSpinner.Value) + 1;

            var allWeightingsProvided = weightings.All(kvp => kvp.Value.HasValue);
            var partialWeightings = weightings.Where(kvp => kvp.Value.HasValue)
                                              .ToDictionary(kvp => kvp.Key,
                                                            kvp => kvp.Value.Value);
            var completeWeightings = partialWeightings.Values.ToArray();
            var useDeafultWeighting = randomDefaultWeightingCheckBox.Checked;
            var defaultWeighting = Convert.ToInt32(randomDefaultWeightingSpinner.Value);

            const int generationCount = 1000000;
            var rng = new Random();

            for (var i = 0; i < generationCount; i++)
            {
                var value = GetRandomNumber(rng,
                                            useMinValue,
                                            minValue,
                                            maxValue,
                                            allWeightingsProvided,
                                            partialWeightings,
                                            completeWeightings,
                                            useDeafultWeighting,
                                            defaultWeighting);

                occurrencesByValue[value] += 1;
            }

            foreach (DataGridViewRow row in randomDataGridView.Rows)
            {
                var value = (int) row.Cells[ValueColumn.Index].Value;
                var occurrences = occurrencesByValue[value];

                row.Cells[OccurrencesColumn.Index].Value = occurrences;
                row.Cells[OccurrencesPercentColumn.Index].Value = occurrences.AsPercentageOf(generationCount);
            }
        }

        private void randomDataGridView_CellValidating(object sender, DataGridViewCellValidatingEventArgs e)
        {
            if (e.ColumnIndex == WeightingColumn.Index &&
                !string.IsNullOrEmpty(e.FormattedValue as string) &&
                (!int.TryParse(e.FormattedValue.ToString(), out var weighting) ||
                 weighting < 0))
            {
                e.Cancel = true;
                MessageBox.Show("Please enter a non-negative integer.",
                                "Invalid Weighting",
                                MessageBoxButtons.OK,
                                MessageBoxIcon.Error);
            }
        }

        private void randomDataGridView_CellValidated(object sender, DataGridViewCellEventArgs e)
        {
            if (e.ColumnIndex == WeightingColumn.Index)
            {
                var cell = randomDataGridView[e.ColumnIndex, e.RowIndex];
                var cellValue = cell.Value;

                if (!string.IsNullOrEmpty(cellValue as string))
                {
                    cell.Value = Convert.ToInt32(cellValue);
                }
            }
        }

        private void randomDataGridView_CellValueChanged(object sender, DataGridViewCellEventArgs e)
        {
            if (randomIgnoreCellValueChanged)
            {
                return;
            }

            if (e.ColumnIndex == WeightingColumn.Index)
            {
                var totalWeighting = randomDataGridView.Rows
                                                       .Cast<DataGridViewRow>()
                                                       .Sum(GetWeightingForRow);

                foreach (DataGridViewRow row in randomDataGridView.Rows)
                {
                    var weighting = GetWeightingForRow(row);

                    row.Cells[WeightingPercentColumn.Index].Value = weighting.AsPercentageOf(totalWeighting);
                }
            }
        }

        #endregion Random

        #region String

        private void InitStringPage()
        {
            stringSubstringSourceTextBox.Text = "abcdefghijklmnopqrstuvwxyz";

            stringTitleCaseInputTextBox.Text = "the QUICK brown Fox jUmPeD OvEr the lazy d0g";
            stringTitleCaseCultureComboBox.DisplayMember = nameof(CultureInfo.DisplayName);
            stringTitleCaseCultureComboBox.DataSource = CultureInfo.GetCultures(CultureTypes.AllCultures);
        }

        #region Convert to Title Case

        private void stringTitleCaseCurrentButton_Click(object sender, EventArgs e)
        {
            stringTitleCaseOutputTextBox.Text = stringTitleCaseAcronymsCheckBox.Checked
                                                    ? stringTitleCaseInputTextBox.Text.ToTitle(true)
                                                    : stringTitleCaseInputTextBox.Text.ToTitle();
        }

        private void stringTitleCaseSelectedButton_Click(object sender, EventArgs e)
        {
            var culture = (CultureInfo) stringTitleCaseCultureComboBox.SelectedItem;

            stringTitleCaseOutputTextBox.Text = stringTitleCaseAcronymsCheckBox.Checked
                                                    ? stringTitleCaseInputTextBox.Text.ToTitle(culture, true)
                                                    : stringTitleCaseInputTextBox.Text.ToTitle(culture);
        }

        private void stringTitleCaseInvariantButton_Click(object sender, EventArgs e)
        {
            stringTitleCaseOutputTextBox.Text = stringTitleCaseAcronymsCheckBox.Checked
                                                    ? stringTitleCaseInputTextBox.Text.ToTitleInvariant(true)
                                                    : stringTitleCaseInputTextBox.Text.ToTitleInvariant();
        }

        #endregion Convert to Title Case

        #region Substring

        private void stringSubstringByLengthButton_Click(object sender, EventArgs e)
        {
            try
            {
                stringSubstringOutputListBox.DataSource = stringSubstringSourceTextBox.Text.SubstringsByLength(Convert.ToInt32(stringSubstringLengthSpinner.Value));
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, nameof(StringExtensions.SubstringsByLength), MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void stringSubstringsByCountButton_Click(object sender, EventArgs e)
        {
            try
            {
                stringSubstringOutputListBox.DataSource = stringSubstringSourceTextBox.Text.SubstringsByCount(Convert.ToInt32(stringSubstringCountSpinner.Value));
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, nameof(StringExtensions.SubstringsByCount), MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        #endregion Substring

        #endregion String
    }
}
