﻿using System.Collections.Generic;
using System.Drawing;
using System.IO;
using System.Linq;

namespace Wunnell.Extensions
{
    /// <summary>
    /// Contains methods that extend the <see cref="IEnumerable{T}"/> interface.
    /// </summary>
    public static class EnumerableExtensions
    {
        /// <summary>
        /// Converts an array of <see cref="byte"/> containing image data into an <see cref="Image"/> object.
        /// </summary>
        /// <param name="source">
        /// The input byte array.
        /// </param>
        /// <returns>
        /// The image represented by the input data.
        /// </returns>
        public static Image ToImage(this IEnumerable<byte> source)
        {
            using (var stream = new MemoryStream(source.ToArray()))
            {
                return Image.FromStream(stream);
            }
        }
    }
}
