﻿using System;
using System.Drawing;
using System.Drawing.Drawing2D;

namespace Wunnell.Extensions
{
    /// <summary>
    /// Contains methods that extend the <see cref="Graphics"/> class.
    /// </summary>
    public static class GraphicsExtensions
    {
        public static void DrawCircle(this Graphics source, Pen pen, Point center, int radius)
        {
            var diameter = radius * 2;
            var rect = new Rectangle(center.X - radius,
                                     center.Y - radius,
                                     diameter,
                                     diameter);

            source.DrawEllipse(pen, rect);
        }

        public static void DrawCircle(this Graphics source, Pen pen, PointF center, float radius)
        {
            var diameter = radius * 2;
            var rect = new RectangleF(center.X - radius,
                                      center.Y - radius,
                                      diameter,
                                      diameter);

            source.DrawEllipse(pen, rect);
        }

        public static void FillCircle(this Graphics source, Brush brush, Point center, int radius)
        {
            var diameter = radius * 2;
            var rect = new Rectangle(center.X - radius,
                                     center.Y - radius,
                                     diameter,
                                     diameter);

            source.FillEllipse(brush, rect);
        }

        public static void FillCircle(this Graphics source, Brush brush, PointF center, float radius)
        {
            var diameter = radius * 2;
            var rect = new RectangleF(center.X - radius,
                                      center.Y - radius,
                                      diameter,
                                      diameter);

            source.FillEllipse(brush, rect);
        }

        public static void DrawTriangle(this Graphics source, Pen pen, Point[] vertices)
        {
            if (vertices.Length != 3)
            {
                throw new ArgumentException("Exactly three vertices must be provided.", nameof(vertices));
            }

            var path = new GraphicsPath();

            path.AddLines(vertices);
            path.CloseFigure();

            source.DrawPath(pen, path);
        }

        public static void DrawTriangle(this Graphics source, Pen pen, PointF[] vertices)
        {
            if (vertices.Length != 3)
            {
                throw new ArgumentException("Exactly three vertices must be provided.", nameof(vertices));
            }

            var path = new GraphicsPath();

            path.AddLines(vertices);
            path.CloseFigure();

            source.DrawPath(pen, path);
        }

        public static void FillTriangle(this Graphics source, Brush brush, Point[] vertices)
        {
            if (vertices.Length != 3)
            {
                throw new ArgumentException("Exactly three vertices must be provided.", nameof(vertices));
            }

            var path = new GraphicsPath();

            path.AddLines(vertices);
            path.CloseFigure();

            source.FillPath(brush, path);
        }

        public static void FillTriangle(this Graphics source, Brush brush, PointF[] vertices)
        {
            if (vertices.Length != 3)
            {
                throw new ArgumentException("Exactly three vertices must be provided.", nameof(vertices));
            }

            var path = new GraphicsPath();

            path.AddLines(vertices);
            path.CloseFigure();

            source.FillPath(brush, path);
        }
    }
}
