﻿using System.Drawing;
using System.Drawing.Imaging;
using System.IO;

namespace Wunnell.Extensions
{
    /// <summary>
    /// Contains methods that extend the <see cref="Image"/> class.
    /// </summary>
    public static class ImageExtensions
    {
        /// <summary>
        /// Saves an <see cref="Image"/> to an array of <see cref="byte"/>'s.
        /// </summary>
        /// <param name="source">
        /// The inoput image.
        /// </param>
        /// <returns>
        /// An array containing the image data.
        /// </returns>
        public static byte[] ToBytes(this Image source)
        {
            return source.ToBytes(source.RawFormat);
        }

        /// <summary>
        /// Saves an <see cref="Image"/> to an array of <see cref="byte"/>'s in the specified format.
        /// </summary>
        /// <param name="source">
        /// The inoput image.
        /// </param>
        /// <param name="format">
        /// The <see cref="ImageFormat"/> for this <see cref="Image"/>.
        /// </param>
        /// <returns>
        /// An array containing the image data.
        /// </returns>
        public static byte[] ToBytes(this Image source, ImageFormat format)
        {
            using (var stream = new MemoryStream())
            {
                source.Save(stream, format);

                return stream.ToArray();
            }
        }

        /// <summary>
        /// Saves an <see cref="Image"/> to an array of <see cref="byte"/>'s, with the specified encoder and image-encoder parameters.
        /// </summary>
        /// <param name="source">
        /// The inoput image.
        /// </param>
        /// <param name="encoder"></param>
        /// <param name="encoderParams"></param>
        /// <returns>
        /// An array containing the image data.
        /// </returns>
        public static byte[] ToBytes(this Image source, ImageCodecInfo encoder, EncoderParameters encoderParams)
        {
            using (var stream = new MemoryStream())
            {
                source.Save(stream, encoder, encoderParams);

                return stream.ToArray();
            }
        }
    }
}
