﻿Imports System.Drawing
Imports System.Drawing.Drawing2D
Imports System.Runtime.CompilerServices

''' <summary>
''' Contains methods that extend the <see cref="Graphics"/> class.
''' </summary>
Public Module GraphicsExtensions

    <Extension>
    Public Sub DrawCircle(source As Graphics, pen As Pen, center As Point, radius As Integer)
        Dim diameter = radius * 2
        Dim rect As New Rectangle(center.X - radius,
                                  center.Y - radius,
                                  diameter,
                                  diameter)

        source.DrawEllipse(pen, rect)
    End Sub

    <Extension>
    Public Sub DrawCircle(source As Graphics, pen As Pen, center As PointF, radius As Single)
        Dim diameter = radius * 2
        Dim rect As New RectangleF(center.X - radius,
                                   center.Y - radius,
                                   diameter,
                                   diameter)

        source.DrawEllipse(pen, rect)
    End Sub

    <Extension>
    Public Sub FillCircle(source As Graphics, brush As Brush, center As Point, radius As Integer)
        Dim diameter = radius * 2
        Dim rect As New Rectangle(center.X - radius,
                                  center.Y - radius,
                                  diameter,
                                  diameter)

        source.FillEllipse(brush, rect)
    End Sub

    <Extension>
    Public Sub FillCircle(source As Graphics, brush As Brush, center As PointF, radius As Single)
        Dim diameter = radius * 2
        Dim rect As New RectangleF(center.X - radius,
                                   center.Y - radius,
                                   diameter,
                                   diameter)

        source.FillEllipse(brush, rect)
    End Sub

    <Extension>
    Public Sub DrawTriangle(source As Graphics, pen As Pen, vertices As Point())
        If vertices.Length <> 3 Then
            Throw New ArgumentException("Exactly three vertices must be provided.", NameOf(vertices))
        End If

        Dim path As New GraphicsPath

        path.AddLines(vertices)
        path.CloseFigure()

        source.DrawPath(pen, path)
    End Sub

    <Extension>
    Public Sub DrawTriangle(source As Graphics, pen As Pen, vertices As PointF())
        If vertices.Length <> 3 Then
            Throw New ArgumentException("Exactly three vertices must be provided.", NameOf(vertices))
        End If

        Dim path As New GraphicsPath

        path.AddLines(vertices)
        path.CloseFigure()

        source.DrawPath(pen, path)
    End Sub

    <Extension>
    Public Sub FillTriangle(source As Graphics, brush As Brush, vertices As Point())
        If vertices.Length <> 3 Then
            Throw New ArgumentException("Exactly three vertices must be provided.", NameOf(vertices))
        End If

        Dim path As New GraphicsPath

        path.AddLines(vertices)
        path.CloseFigure()

        source.FillPath(brush, path)
    End Sub

    <Extension>
    Public Sub FillTriangle(source As Graphics, brush As Brush, vertices As PointF())
        If vertices.Length <> 3 Then
            Throw New ArgumentException("Exactly three vertices must be provided.", NameOf(vertices))
        End If

        Dim path As New GraphicsPath

        path.AddLines(vertices)
        path.CloseFigure()

        source.FillPath(brush, path)
    End Sub

End Module
