﻿Imports System.Drawing
Imports System.Drawing.Imaging
Imports System.IO
Imports System.Runtime.CompilerServices

''' <summary>
''' Contains methods that extend the <see cref="Image"/> class.
''' </summary>
Public Module ImageExtensions

    ''' <summary>
    ''' Saves an <see cref="Image"/> to an array of <see cref="Byte"/>'s.
    ''' </summary>
    ''' <param name="source">
    ''' The inoput image.
    ''' </param>
    ''' <returns>
    ''' An array containing the image data.
    ''' </returns>
    <Extension>
    Public Function ToBytes(source As Image) As Byte()
        Return source.ToBytes(source.RawFormat)
    End Function

    ''' <summary>
    ''' Saves an <see cref="Image"/> to an array of <see cref="Byte"/>'s in the specified format.
    ''' </summary>
    ''' <param name="source">
    ''' The inoput image.
    ''' </param>
    ''' <param name="format">
    ''' The <see cref="ImageFormat"/> for this <see cref="Image"/>.
    ''' </param>
    ''' <returns>
    ''' An array containing the image data.
    ''' </returns>
    <Extension>
    Public Function ToBytes(source As Image, format As ImageFormat) As Byte()
        Using stream As New MemoryStream
            source.Save(stream, format)

            Return stream.ToArray()
        End Using
    End Function

    ''' <summary>
    ''' Saves an <see cref="Image"/> to an array of <see cref="Byte"/>'s, with the specified encoder and image-encoder parameters.
    ''' </summary>
    ''' <param name="source">
    ''' The inoput image.
    ''' </param>
    ''' <param name="encoder">
    ''' The <see cref="ImageCodecInfo"/> for this <see cref="Image"/>.
    ''' </param>
    ''' <param name="encoderParams">
    ''' An <see cref="EncoderParameters"/> to use for this <see cref="Image"/>.
    ''' </param>
    ''' <returns>
    ''' An array containing the image data.
    ''' </returns>
    <Extension>
    Public Function ToBytes(source As Image, encoder As ImageCodecInfo, encoderParams As EncoderParameters) As Byte()
        Using stream As New MemoryStream
            source.Save(stream, encoder, encoderParams)

            Return stream.ToArray()
        End Using
    End Function

End Module
