﻿using System;

namespace Wunnell.Extensions
{
    public static class ArrayExtensions
    {
        public static void ShiftLeft<T>(this T[] source)
        {
            source.ShiftLeft(1);
        }

        public static void ShiftLeft<T>(this T[] source, int count)
        {
            var sourceLength = source.Length;

            if (sourceLength > 1)
            {
                // If the shift wraps completely one or more times, ignore all but the remaining partial shift.
                count = count % sourceLength;

                if (count > 0)
                {
                    var part1StartIndex = 0;
                    var part1EndIndex = count - 1;
                    var part1Length = count;
                    var part2StartIndex = count;
                    var part2EndIndex = source.GetUpperBound(0);
                    var part2Length = sourceLength - count;

                    // One part of the array will need to be copied while the other is shifted.
                    // Copy the smaller part for efficiency.
                    // NOTE: In comments below, N is the length of the source array and C is the count of places to shift.
                    if (part1Length <= part2Length)
                    {
                        var temp = new T[part1Length];

                        // Copy C elements from the beginning of the array.
                        Array.Copy(source, temp, part1Length);

                        // Shift N - C elements from the end of the array C places forward.
                        for (var i = part2StartIndex; i <= part2EndIndex; i++)
                        {
                            source[i - part1Length] = source[i];
                        }

                        // Copy C elements to the end of the array.
                        Array.Copy(temp, 0, source, sourceLength - part1Length, part1Length);
                    }
                    else
                    {
                        var temp = new T[part2Length];

                        // Copy N - C elements from the end of the array.
                        Array.Copy(source, part1Length, temp, 0, part2Length);

                        // Shift C elements from the beginning of the array N - C places backward.
                        for (var i = part1EndIndex; i >= part1StartIndex; i--)
                        {
                            source[i + part2Length] = source[i];
                        }

                        // Copy N - C elements to the beginning of the array.
                        Array.Copy(temp, source, part2Length);
                    }
                }
            }
        }

        public static void ShiftRight<T>(this T[] source)
        {
            source.ShiftRight(1);
        }

        public static void ShiftRight<T>(this T[] source, int count)
        {
            var sourceLength = source.Length;

            if (sourceLength > 1)
            {
                // If the shift wraps completely one or more times, ignore all but the remaining partial shift.
                count = count % sourceLength;

                if (count > 0)
                {
                    var part1StartIndex = 0;
                    var part1EndIndex = sourceLength - count - 1;
                    var part1Length = sourceLength - count;
                    var part2StartIndex = sourceLength - count;
                    var part2EndIndex = source.GetUpperBound(0);
                    var part2Length = count;

                    // One part of the array will need to be copied while the other is shifted.
                    // Copy the smaller part for efficiency.
                    // NOTE: In comments below, N is the length of the source array and C is the count of places to shift.
                    if (part1Length <= part2Length)
                    {
                        var temp = new T[part1Length];

                        // Copy N-C elements from the beginning of the array.
                        Array.Copy(source, temp, part1Length);

                        // Shift C elements from the end of the array N-C places forward.
                        for (var i = part2StartIndex; i <= part2EndIndex; i++)
                        {
                            source[i - part1Length] = source[i];
                        }

                        // Copy N-C elements to the end of the array.
                        Array.Copy(temp, 0, source, sourceLength - part1Length, part1Length);
                    }
                    else
                    {
                        var temp = new T[part2Length];

                        // Copy C elements from the end of the array.
                        Array.Copy(source, part1Length, temp, 0, part2Length);

                        // Shift N-C elements from the beginning of the array C places backward.
                        for (var i = part1EndIndex; i >= part1StartIndex; i--)
                        {
                            source[i + part2Length] = source[i];
                        }

                        // Copy C elements to the beginning of the array.
                        Array.Copy(temp, source, part2Length);
                    }
                }
            }
        }

        public static T GetNext<T>(this T[] source, ref int index)
        {
            index++;

            if (index < source.GetLowerBound(0) || index > source.GetUpperBound(0))
            {
                index = source.GetLowerBound(0);
            }

            return source[index];
        }

        public static T GetPrevious<T>(this T[] source, ref int index)
        {
            index--;

            if (index < source.GetLowerBound(0) || index > source.GetUpperBound(0))
            {
                index = source.GetUpperBound(0);
            }

            return source[index];
        }
    }
}