﻿Imports System.Runtime.CompilerServices

Public Module ArrayExtensions

    <Extension>
    Public Sub ShiftLeft(Of T)(source As T())
        source.ShiftLeft(1)
    End Sub

    <Extension>
    Public Sub ShiftLeft(Of T)(source As T(), count As Integer)
        Dim sourceLength = source.Length

        If sourceLength > 1 Then
            'If the shift wraps completely one or more times, ignore all but the remaining partial shift.
            count = count Mod sourceLength

            If count > 0 Then
                Dim part1StartIndex = 0
                Dim part1EndIndex = count - 1
                Dim part1Length = count
                Dim part2StartIndex = count
                Dim part2EndIndex = source.GetUpperBound(0)
                Dim part2Length = sourceLength - count

                'One part of the array will need to be copied while the other is shifted.
                'Copy the smaller part for efficiency.
                'NOTE: In comments below, N is the length of the source array and C is the count of places to shift.
                If part1Length <= part2Length Then
                    Dim temp(part1Length - 1) As T

                    'Copy C elements from the beginning of the array.
                    Array.Copy(source, temp, part1Length)

                    'Shift N-C elements from the end of the array C places forward.
                    For i = part2StartIndex To part2EndIndex
                        source(i - part1Length) = source(i)
                    Next

                    'Copy C elements to the end of the array.
                    Array.Copy(temp, 0, source, sourceLength - part1Length, part1Length)
                Else
                    Dim temp(part2Length - 1) As T

                    'Copy N-C elements from the end of the array.
                    Array.Copy(source, part1Length, temp, 0, part2Length)

                    'Shift C elements from the beginning of the array N-C places backward.
                    For i = part1EndIndex To part1StartIndex Step -1
                        source(i + part2Length) = source(i)
                    Next

                    'Copy N-C elements to the beginning of the array.
                    Array.Copy(temp, source, part2Length)
                End If
            End If
        End If
    End Sub

    <Extension>
    Public Sub ShiftRight(Of T)(source As T())
        source.ShiftRight(1)
    End Sub

    <Extension>
    Public Sub ShiftRight(Of T)(source As T(), count As Integer)
        Dim sourceLength = source.Length

        If sourceLength > 1 Then
            'If the shift wraps completely one or more times, ignore all but the remaining partial shift.
            count = count Mod sourceLength

            If count > 0 Then
                Dim part1StartIndex = 0
                Dim part1EndIndex = sourceLength - count - 1
                Dim part1Length = sourceLength - count
                Dim part2StartIndex = sourceLength - count
                Dim part2EndIndex = source.GetUpperBound(0)
                Dim part2Length = count

                'One part of the array will need to be copied while the other is shifted.
                'Copy the smaller part for efficiency.
                'NOTE: In comments below, N is the length of the source array and C is the count of places to shift.
                If part1Length <= part2Length Then
                    Dim temp(part1Length - 1) As T

                    'Copy N-C elements from the beginning of the array.
                    Array.Copy(source, temp, part1Length)

                    'Shift C elements from the end of the array N-C places forward.
                    For i = part2StartIndex To part2EndIndex
                        source(i - part1Length) = source(i)
                    Next

                    'Copy N-C elements to the end of the array.
                    Array.Copy(temp, 0, source, sourceLength - part1Length, part1Length)
                Else
                    Dim temp(part2Length - 1) As T

                    'Copy C elements from the end of the array.
                    Array.Copy(source, part1Length, temp, 0, part2Length)

                    'Shift N-C elements from the beginning of the array C places backward.
                    For i = part1EndIndex To part1StartIndex Step -1
                        source(i + part2Length) = source(i)
                    Next

                    'Copy C elements to the beginning of the array.
                    Array.Copy(temp, source, part2Length)
                End If
            End If
        End If
    End Sub

    <Extension>
    Public Function GetNext(Of T)(source As T(), ByRef index As Integer) As T
        index += 1

        If index < source.GetLowerBound(0) OrElse index > source.GetUpperBound(0) Then
            index = source.GetLowerBound(0)
        End If

        Return source(index)
    End Function

    <Extension>
    Public Function GetPrevious(Of T)(source As T(), ByRef index As Integer) As T
        index -= 1

        If index < source.GetLowerBound(0) OrElse index > source.GetUpperBound(0) Then
            index = source.GetUpperBound(0)
        End If

        Return source(index)
    End Function

End Module
