﻿Imports System.Runtime.CompilerServices

''' <summary>
''' Contains methods that extend the <see cref="IEnumerable(Of T)"/> interface.
''' </summary>
Public Module EnumerableExtensions

    ''' <summary>
    ''' A random number generator.
    ''' </summary>
    Private rng As Random

    ''' <summary>
    ''' Randomises the items in an enumerable list.
    ''' </summary>
    ''' <typeparam name="T">
    ''' The type of the items in the list.
    ''' </typeparam>
    ''' <param name="source">
    ''' The input list.
    ''' </param>
    ''' <returns>
    ''' The items from the input list in random order.
    ''' </returns>
    <Extension>
    Public Function Randomize(Of T)(source As IEnumerable(Of T)) As IEnumerable(Of T)
        EnsureRandomInitialized()

        Return source.Randomize(rng)
    End Function

    ''' <summary>
    ''' Randomises the items in an enumerable list.
    ''' </summary>
    ''' <typeparam name="T">
    ''' The type of the items in the list.
    ''' </typeparam>
    ''' <param name="source">
    ''' The input list.
    ''' </param>
    ''' <param name="random">
    ''' A random number generator.
    ''' </param>
    ''' <returns>
    ''' The items from the input list in random order.
    ''' </returns>
    <Extension>
    Public Function Randomize(Of T)(source As IEnumerable(Of T), random As Random) As IEnumerable(Of T)
        Return source.OrderBy(Function(o) random.NextDouble())
    End Function

    ''' <summary>
    ''' Initialises the random number generator if it is not already.
    ''' </summary>
    Private Sub EnsureRandomInitialized()
        rng = If(rng, New Random)
    End Sub

End Module
