﻿using System;

namespace Wunnell.Extensions
{
    /// <summary>
    /// Contains methods that extend the <see cref="Decimal"/> structure.
    /// </summary>
    public static class DecimalExtensions
    {
        /// <summary>
        /// Calculates what percentage of a specified value the input is.
        /// </summary>
        /// <param name="source">
        /// The input number.
        /// </param>
        /// <param name="value">
        /// The number to calculate the percentage of.
        /// </param>
        /// <returns>
        /// The percentage of <i>value</i> that the input represents.
        /// </returns>
        public static decimal AsPercentageOf(this decimal source, decimal value)
        {
            return 100M * source / value;
        }

        /// <summary>
        /// Calculates a proportion of a specified value determined by the input as a percentage.
        /// </summary>
        /// <param name="source">
        /// The input number.
        /// </param>
        /// <param name="value">
        /// The number to calculate the percentage of.
        /// </param>
        /// <returns>
        /// A value that is the percentage of <i>value</i> specified by the input.
        /// </returns>
        public static decimal PercentOf(this decimal source, decimal value)
        {
            return source * value / 100M;
        }
    }
}
