﻿using System;

namespace Wunnell.Extensions
{
    /// <summary>
    /// Contains methods that extend the <see cref="Double"/> structure.
    /// </summary>
    public static class DoubleExtensions
    {
        /// <summary>
        /// Determines whether two <see cref="Double"/> values are equal to within a specified tolerance.
        /// </summary>
        /// <param name="source">
        /// The input number.
        /// </param>
        /// <param name="value">
        /// The number to compare to.
        /// </param>
        /// <param name="tolerance">
        /// The margin for error.
        /// </param>
        /// <returns>
        /// <b>true</b> if the two values are within the specific tolerance of each other; otherwise, <b>false</b>.
        /// </returns>
        public static bool IsEquivalentTo(this double source, double value, double tolerance)
        {
            return Math.Abs(source - value) <= tolerance;
        }

        /// <summary>
        /// Calculates what percentage of a specified value the input is.
        /// </summary>
        /// <param name="source">
        /// The input number.
        /// </param>
        /// <param name="value">
        /// The number to calculate the percentage of.
        /// </param>
        /// <returns>
        /// The percentage of <i>value</i> that the input represents.
        /// </returns>
        public static double AsPercentageOf(this double source, double value)
        {
            return 100.0 * source / value;
        }

        /// <summary>
        /// Calculates a proportion of a specified value determined by the input as a percentage.
        /// </summary>
        /// <param name="source">
        /// The input number.
        /// </param>
        /// <param name="value">
        /// The number to calculate the percentage of.
        /// </param>
        /// <returns>
        /// A value that is the percentage of <i>value</i> specified by the input.
        /// </returns>
        public static double PercentOf(this double source, double value)
        {
            return source * value / 100.0;
        }
    }
}
