﻿Imports System.Runtime.CompilerServices

''' <summary>
''' Contains methods that extend the <see cref="Double"/> structure.
''' </summary>
Public Module DoubleExtensions

    ''' <summary>
    ''' Determines whether two <see cref="Double"/> values are equal to within a specified tolerance.
    ''' </summary>
    ''' <param name="source">
    ''' The input number.
    ''' </param>
    ''' <param name="value">
    ''' The number to compare to.
    ''' </param>
    ''' <param name="tolerance">
    ''' The margin for error.
    ''' </param>
    ''' <returns>
    ''' <b>true</b> if the two values are within the specific tolerance of each other; otherwise, <b>false</b>.
    ''' </returns>
    <Extension>
    Public Function IsEquivalentTo(source As Double, value As Double, tolerance As Double) As Boolean
        Return Math.Abs(source - value) <= tolerance
    End Function

    ''' <summary>
    ''' Calculates what percentage of a specified value the input Is.
    ''' </summary>
    ''' <param name="source">
    ''' The input number.
    ''' </param>
    ''' <param name="value">
    ''' The number to calculate the percentage of.
    ''' </param>
    ''' <returns>
    ''' The percentage of <i>value</i> that the input represents.
    ''' </returns>
    <Extension>
    Public Function AsPercentageOf(source As Double, value As Double) As Double
        Return 100.0 * source / value
    End Function

    ''' <summary>
    ''' Calculates a proportion of a specified value determined by the input as a percentage.
    ''' </summary>
    ''' <param name="source">
    ''' The input number.
    ''' </param>
    ''' <param name="value">
    ''' The number to calculate the percentage of.
    ''' </param>
    ''' <returns>
    ''' A value that is the percentage of <i>value</i> specified by the input.
    ''' </returns>
    <Extension>
    Public Function PercentOf(source As Double, value As Double) As Double
        Return source * value / 100.0
    End Function

End Module
