﻿Imports System.Runtime.CompilerServices

''' <summary>
''' Contains methods that extend the <see cref="Int32"/> structure.
''' </summary>
Public Module Int32Extensions

    ''' <summary>
    ''' Calculates the factorial of a number.
    ''' </summary>
    ''' <param name="source">
    ''' The input number.
    ''' </param>
    ''' <returns>
    ''' 1 if the input Is 0; otherwise the product of the series {1, 2, 3, ..., n} where <i>n</i> Is the input.
    ''' </returns>
    <Extension>
    Public Function Factorial(source As Integer) As Integer
        If source < 0 Then
            Throw New ArgumentOutOfRangeException(NameOf(source), "Value cannot be negative.")
        End If

        If source = 0 Then
            '0! Is 1 by definition.
            Return 1
        End If

        Dim product = 1

        For i = 1 To source
            product *= i
        Next

        Return product
    End Function

    ''' <summary>
    ''' Calculates the <i>nth</i> triangular number.
    ''' </summary>
    ''' <param name="source">
    ''' The input number.
    ''' </param>
    ''' <returns>
    ''' The sum of the series {0, 1, 2, ..., n} where <i>n</i> is the input.
    ''' </returns>
    <Extension>
    Public Function Triangular(source As Integer) As Integer
        If source < 0 Then
            Throw New ArgumentOutOfRangeException(NameOf(source), "Value cannot be negative.")
        End If

        Dim sum = 0

        For i = 0 To source
            sum += i
        Next

        Return sum
    End Function

    ''' <summary>
    ''' Calculates what percentage of a specified value the input Is.
    ''' </summary>
    ''' <param name="source">
    ''' The input number.
    ''' </param>
    ''' <param name="value">
    ''' The number to calculate the percentage of.
    ''' </param>
    ''' <returns>
    ''' The percentage of <i>value</i> that the input represents.
    ''' </returns>
    <Extension>
    Public Function AsPercentageOf(source As Integer, value As Integer) As Double
        Return Convert.ToDouble(source).AsPercentageOf(Convert.ToDouble(value))
    End Function

    ''' <summary>
    ''' Calculates what percentage of a specified value the input Is.
    ''' </summary>
    ''' <param name="source">
    ''' The input number.
    ''' </param>
    ''' <param name="value">
    ''' The number to calculate the percentage of.
    ''' </param>
    ''' <returns>
    ''' The percentage of <i>value</i> that the input represents.
    ''' </returns>
    <Extension>
    Public Function AsPercentageOf(source As Integer, value As Double) As Double
        Return Convert.ToDouble(source).AsPercentageOf(value)
    End Function

    ''' <summary>
    ''' Calculates a proportion of a specified value determined by the input as a percentage.
    ''' </summary>
    ''' <param name="source">
    ''' The input number.
    ''' </param>
    ''' <param name="value">
    ''' The number to calculate the percentage of.
    ''' </param>
    ''' <returns>
    ''' A value that is the percentage of <i>value</i> specified by the input.
    ''' </returns>
    <Extension>
    Public Function PercentOf(source As Integer, value As Integer) As Double
        Return Convert.ToDouble(source).AsPercentageOf(Convert.ToDouble(value))
    End Function

    ''' <summary>
    ''' Calculates a proportion of a specified value determined by the input as a percentage.
    ''' </summary>
    ''' <param name="source">
    ''' The input number.
    ''' </param>
    ''' <param name="value">
    ''' The number to calculate the percentage of.
    ''' </param>
    ''' <returns>
    ''' A value that is the percentage of <i>value</i> specified by the input.
    ''' </returns>
    <Extension>
    Public Function PercentOf(source As Integer, value As Double) As Double
        Return Convert.ToDouble(source).AsPercentageOf(value)
    End Function

End Module
