﻿using System;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace Wunnell.Extensions.Tests.NetFx.CS
{
    [TestClass]
    public class ArrayExtensionsTest
    {
        #region ShiftLeft

        [TestMethod]
        public void ShiftLeftDoesNothingOnEmptyReferenceArray()
        {
            // Arrange
            var source = new string[] { };

            // Act
            source.ShiftLeft();

            // Assert
            Assert.AreEqual(source.Length, 0);
        }

        [TestMethod]
        public void ShiftLeftDoesNothingOnEmptyValueArray()
        {
            // Arrange
            var source = new int[] { };

            // Act
            source.ShiftLeft();

            // Assert
            Assert.AreEqual(source.Length, 0);
        }

        [TestMethod]
        public void ShiftLeftDoesNothingOnReferenceArrayWithOneElement()
        {
            // Arrange
            var element = "Hello World";
            var source = new[] { element };

            // Act
            source.ShiftLeft();

            // Assert
            Assert.AreEqual(source[0], element);
        }

        [TestMethod]
        public void ShiftLeftDoesNothingOnValueArrayWithOneElement()
        {
            // Arrange
            var element = 123;
            var source = new[] { element };

            // Act
            source.ShiftLeft();

            // Assert
            Assert.AreEqual(source[0], element);
        }

        [TestMethod]
        public void ShiftLeftShiftsByOneOnReferenceArray()
        {
            // Arrange;
            var original = new[] { "One", "Two", "Three", "Four", "Five" };
            var source = original.ToArray();

            // Act
            source.ShiftLeft();

            // Assert
            var length = source.Length;

            for (var i = 0; i < source.Length; i++)
            {
                Assert.AreEqual(source[i], original[(i + 1) % length]);
            }
        }

        [TestMethod]
        public void ShiftLeftShiftsByOneOnValueArray()
        {
            // Arrange;
            var original = new[] {1, 2, 3, 4, 5};
            var source = original.ToArray();

            // Act
            source.ShiftLeft();

            // Assert
            var length = source.Length;

            for (var i = 0; i < source.Length; i++)
            {
                Assert.AreEqual(source[i], original[(i + 1) % length]);
            }
        }

        [TestMethod]
        public void ShiftLeftShiftsByCountLessThanLengthOnReferenceArray()
        {
            // Arrange
            var original = new[] { "One", "Two", "Three", "Four", "Five" };
            var source = original.ToArray();
            var count = 3;

            // Act
            source.ShiftLeft(count);

            // Assert
            var length = source.Length;

            for (var i = 0; i < source.Length; i++)
            {
                Assert.AreEqual(source[i], original[(i + count) % length]);
            }
        }

        [TestMethod]
        public void ShiftLeftShiftsByCountLessThanLengthOnValueArray()
        {
            // Arrange
            var original = new[] { 1, 2, 3, 4, 5 };
            var source = original.ToArray();
            var count = 3;

            // Act
            source.ShiftLeft(count);

            // Assert
            var length = source.Length;

            for (var i = 0; i < source.Length; i++)
            {
                Assert.AreEqual(source[i], original[(i + count) % length]);
            }
        }

        [TestMethod]
        public void ShiftLeftShiftsByCountEqualToLengthOnReferenceArray()
        {
            // Arrange
            var original = new[] { "One", "Two", "Three", "Four", "Five" };
            var source = original.ToArray();
            var length = source.Length;

            // Act
            source.ShiftLeft(length);

            // Assert
            for (var i = 0; i < source.Length; i++)
            {
                Assert.AreEqual(source[i], original[(i + length) % length]);
            }
        }

        [TestMethod]
        public void ShiftLeftShiftsByCountEqualToLengthOnValueArray()
        {
            // Arrange
            var original = new[] { 1, 2, 3, 4, 5 };
            var source = original.ToArray();
            var length = source.Length;

            // Act
            source.ShiftLeft(length);

            // Assert
            for (var i = 0; i < source.Length; i++)
            {
                Assert.AreEqual(source[i], original[(i + length) % length]);
            }
        }

        [TestMethod]
        public void ShiftLeftShiftsByCountGreaterThanLengthOnReferenceArray()
        {
            // Arrange
            var original = new[] { "One", "Two", "Three", "Four", "Five" };
            var source = original.ToArray();
            var count = 13;

            // Act
            source.ShiftLeft(count);

            // Assert
            var length = source.Length;

            for (var i = 0; i < source.Length; i++)
            {
                Assert.AreEqual(source[i], original[(i + count) % length]);
            }
        }

        [TestMethod]
        public void ShiftLeftShiftsByCountGreaterThanLengthOnValueArray()
        {
            // Arrange
            var original = new[] { 1, 2, 3, 4, 5 };
            var source = original.ToArray();
            var count = 3;

            // Act
            source.ShiftLeft(count);

            // Assert
            var length = source.Length;

            for (var i = 0; i < source.Length; i++)
            {
                Assert.AreEqual(source[i], original[(i + count) % length]);
            }
        }

        [TestMethod]
        public void ShiftLeftByMultipleMatchesMultipleShiftLeftsOnReferenceArray()
        {
            // Arrange
            var original = new[] { "One", "Two", "Three", "Four", "Five" };
            var testCount = 3 * original.Length;
            var shiftsByMultiple = Enumerable.Range(1, testCount).Select(n => original.ToArray()).ToArray();
            var multipleShifts = Enumerable.Range(1, testCount).Select(n => original.ToArray()).ToArray();

            // Act
            for (var i = 0; i < testCount; i++)
            {
                shiftsByMultiple[i].ShiftLeft(i + 1);

                for (var j = 1; j <= i + 1; j++)
                {
                    multipleShifts[i].ShiftLeft();
                }
            }

            // Assert
            var upperBound = original.GetUpperBound(0);

            for (var i = 0; i < testCount; i++)
            {
                for (var j = 0; j <= upperBound; j++)
                {
                    Assert.AreEqual(shiftsByMultiple[i][j], multipleShifts[i][j]);
                }
            }
        }

        [TestMethod]
        public void ShiftLeftByMultipleMatchesMultipleShiftLeftsOnValueArray()
        {
            // Arrange
            var original = new[] { 1, 2, 3, 4, 5 };
            var testCount = 3 * original.Length;
            var shiftsByMultiple = Enumerable.Range(1, testCount).Select(n => original.ToArray()).ToArray();
            var multipleShifts = Enumerable.Range(1, testCount).Select(n => original.ToArray()).ToArray();

            // Act
            for (var i = 0; i < testCount; i++)
            {
                shiftsByMultiple[i].ShiftLeft(i + 1);

                for (var j = 1; j <= i + 1; j++)
                {
                    multipleShifts[i].ShiftLeft();
                }
            }

            // Assert
            var upperBound = original.GetUpperBound(0);

            for (var i = 0; i < testCount; i++)
            {
                for (var j = 0; j <= upperBound; j++)
                {
                    Assert.AreEqual(shiftsByMultiple[i][j], multipleShifts[i][j]);
                }
            }
        }

        #endregion ShiftLeft

        #region ShiftRight

        [TestMethod]
        public void ShiftRightDoesNothingOnEmptyReferenceArray()
        {
            // Arrange

            // Act

            // Assert
        }

        [TestMethod]
        public void ShiftRightDoesNothingOnEmptyValueArray()
        {
            // Arrange

            // Act

            // Assert
        }

        [TestMethod]
        public void ShiftRightDoesNothingOnReferenceArrayWithOneElement()
        {
            // Arrange

            // Act

            // Assert
        }

        [TestMethod]
        public void ShiftRightDoesNothingOnValueArrayWithOneElement()
        {
            // Arrange

            // Act

            // Assert
        }

        [TestMethod]
        public void ShiftRightShiftsByOneOnReferenceArray()
        {
            // Arrange

            // Act

            // Assert
        }

        [TestMethod]
        public void ShiftRightShiftsByOneOnValueArray()
        {
            // Arrange

            // Act

            // Assert
        }

        [TestMethod]
        public void ShiftRightShiftsByCountLessThanLengthOnReferenceArray()
        {
            // Arrange

            // Act

            // Assert
        }

        [TestMethod]
        public void ShiftRightShiftsByCountLessThanLengthOnValueArray()
        {
            // Arrange

            // Act

            // Assert
        }

        [TestMethod]
        public void ShiftRightShiftsByCountEqualToThanLengthOnReferenceArray()
        {
            // Arrange

            // Act

            // Assert
        }

        [TestMethod]
        public void ShiftRightShiftsByCountEqualToLengthOnValueArray()
        {
            // Arrange

            // Act

            // Assert
        }

        [TestMethod]
        public void ShiftRightShiftsByCountGreaterThanLengthOnReferenceArray()
        {
            // Arrange

            // Act

            // Assert
        }

        [TestMethod]
        public void ShiftRightShiftsByCountGreaterThanLengthOnValueArray()
        {
            // Arrange

            // Act

            // Assert
        }

        [TestMethod]
        public void ShiftRightByMultipleMatchesMultipleShiftRightsOnReferenceArray()
        {
            // Arrange

            // Act

            // Assert
        }

        [TestMethod]
        public void ShiftRightByMultipleMatchesMultipleShiftRightsOnValueArray()
        {
            // Arrange

            // Act

            // Assert
        }

        #endregion ShiftRight

        #region GetNext

        [TestMethod]
        public void GetNextThrowsExceptionOnEmptyReferenceArray()
        {
            // Arrange

            // Act

            // Assert
        }

        [TestMethod]
        public void GetNextThrowsExceptionOnEmptyValueArray()
        {
            // Arrange

            // Act

            // Assert
        }

        [TestMethod]
        public void GetNextSucceedsWithoutWrappingOnReferenceArray()
        {
            // Arrange

            // Act

            // Assert
        }

        [TestMethod]
        public void GetNextSucceedsWithoutWrappingOnValueArray()
        {
            // Arrange

            // Act

            // Assert
        }

        [TestMethod]
        public void GetNextSucceedsWithWrappingOnReferenceArray()
        {
            // Arrange

            // Act

            // Assert
        }

        [TestMethod]
        public void GetNextSucceedsWithWrappingOnValueArray()
        {
            // Arrange

            // Act

            // Assert
        }

        #endregion GetNext

        #region GetPrevious

        [TestMethod]
        public void GetPreviousThrowsExceptionOnEmptyReferenceArray()
        {
            // Arrange

            // Act

            // Assert
        }

        [TestMethod]
        public void GetPreviousThrowsExceptionOnEmptyValueArray()
        {
            // Arrange

            // Act

            // Assert
        }

        [TestMethod]
        public void GetPreviousSucceedsWithoutWrappingOnReferenceArray()
        {
            // Arrange

            // Act

            // Assert
        }

        [TestMethod]
        public void GetPreviousSucceedsWithoutWrappingOnValueArray()
        {
            // Arrange

            // Act

            // Assert
        }

        [TestMethod]
        public void GetPreviousSucceedsWithWrappingOnReferenceArray()
        {
            // Arrange

            // Act

            // Assert
        }

        [TestMethod]
        public void GetPreviousSucceedsWithWrappingOnValueArray()
        {
            // Arrange

            // Act

            // Assert
        }

        #endregion GetPrevious
    }
}
