﻿Imports System.Text
Imports Microsoft.VisualStudio.TestTools.UnitTesting

<TestClass>
Public Class ArrayExtensionsTest

#Region "ShiftLeft"

    <TestMethod>
    Public Sub ShiftLeftDoesNothingOnEmptyReferenceArray()
        'Arrange
        Dim source As String() = {}

        'Act
        source.ShiftLeft()

        'Assert
        Assert.AreEqual(source.Length, 0)
    End Sub

    <TestMethod>
    Public Sub ShiftLeftDoesNothingOnEmptyValueArray()
        'Arrange
        Dim source As Integer() = {}

        'Act
        source.ShiftLeft()

        'Assert
        Assert.AreEqual(source.Length, 0)
    End Sub

    <TestMethod>
    Public Sub ShiftLeftDoesNothingOnReferenceArrayWithOneElement()
        'Arrange
        Dim element = "Hello World"
        Dim source As String() = {element}

        'Act
        source.ShiftLeft()

        'Assert
        Assert.AreEqual(source(0), element)
    End Sub

    <TestMethod>
    Public Sub ShiftLeftDoesNothingOnValueArrayWithOneElement()
        'Arrange
        Dim element = 123
        Dim source As Integer() = {element}

        'Act
        source.ShiftLeft()

        'Assert
        Assert.AreEqual(source(0), element)
    End Sub

    <TestMethod>
    Public Sub ShiftLeftShiftsByOneOnReferenceArray()
        'Arrange
        Dim original = {"One", "Two", "Three", "Four", "Five"}
        Dim source = original.ToArray()

        'Act
        source.ShiftLeft()

        'Assert
        Dim length = source.Length

        For i = 0 To source.GetUpperBound(0)
            Assert.AreEqual(source(i), original((i + 1) Mod length))
        Next
    End Sub

    <TestMethod>
    Public Sub ShiftLeftShiftsByOneOnValueArray()
        'Arrange
        Dim original = {1, 2, 3, 4, 5}
        Dim source = original.ToArray()

        'Act
        source.ShiftLeft()

        'Assert
        Dim length = source.Length

        For i = 0 To source.GetUpperBound(0)
            Assert.AreEqual(source(i), original((i + 1) Mod length))
        Next
    End Sub

    <TestMethod>
    Public Sub ShiftLeftShiftsByCountLessThanLengthOnReferenceArray()
        'Arrange
        Dim original = {"One", "Two", "Three", "Four", "Five"}
        Dim source = original.ToArray()
        Dim count = 3

        'Act
        source.ShiftLeft(count)

        'Assert
        Dim length = source.Length

        For i = 0 To source.GetUpperBound(0)
            Assert.AreEqual(source(i), original((i + count) Mod length))
        Next
    End Sub

    <TestMethod>
    Public Sub ShiftLeftShiftsByCountLessThanLengthOnValueArray()
        'Arrange
        Dim original = {1, 2, 3, 4, 5}
        Dim source = original.ToArray()
        Dim count = 3

        'Act
        source.ShiftLeft(count)

        'Assert
        Dim length = source.Length

        For i = 0 To source.GetUpperBound(0)
            Assert.AreEqual(source(i), original((i + count) Mod length))
        Next
    End Sub

    <TestMethod>
    Public Sub ShiftLeftShiftsByCountEqualToLengthOnReferenceArray()
        'Arrange
        Dim original = {"One", "Two", "Three", "Four", "Five"}
        Dim source = original.ToArray()
        Dim length = source.Length

        'Act
        source.ShiftLeft(length)

        'Assert
        For i = 0 To source.GetUpperBound(0)
            Assert.AreEqual(source(i), original((i + length) Mod length))
        Next
    End Sub

    <TestMethod>
    Public Sub ShiftLeftShiftsByCountEqualToLengthOnValueArray()
        'Arrange
        Dim original = {1, 2, 3, 4, 5}
        Dim source = original.ToArray()
        Dim length = source.Length

        'Act
        source.ShiftLeft(length)

        'Assert
        For i = 0 To source.GetUpperBound(0)
            Assert.AreEqual(source(i), original((i + length) Mod length))
        Next
    End Sub

    <TestMethod>
    Public Sub ShiftLeftShiftsByCountGreaterThanLengthOnReferenceArray()
        'Arrange
        Dim original = {"One", "Two", "Three", "Four", "Five"}
        Dim source = original.ToArray()
        Dim count = 13

        'Act
        source.ShiftLeft(count)

        'Assert
        Dim length = source.Length

        For i = 0 To source.GetUpperBound(0)
            Assert.AreEqual(source(i), original((i + count) Mod length))
        Next
    End Sub

    <TestMethod>
    Public Sub ShiftLeftShiftsByCountGreaterThanLengthOnValueArray()
        'Arrange
        Dim original = {1, 2, 3, 4, 5}
        Dim source = original.ToArray()
        Dim count = 13

        'Act
        source.ShiftLeft(count)

        'Assert
        Dim length = source.Length

        For i = 0 To source.GetUpperBound(0)
            Assert.AreEqual(source(i), original((i + count) Mod length))
        Next
    End Sub

    <TestMethod>
    Public Sub ShiftLeftByMultipleMatchesMultipleShiftLeftsOnReferenceArray()
        'Arrange
        Dim original = {"One", "Two", "Three", "Four", "Five"}
        Dim testCount = 3 * original.Length
        Dim shiftsByMultiple = Enumerable.Range(1, testCount).Select(Function(n) original.ToArray()).ToArray()
        Dim multipleShifts = Enumerable.Range(1, testCount).Select(Function(n) original.ToArray()).ToArray()

        'Act
        For i = 0 To testCount - 1
            shiftsByMultiple(i).ShiftLeft(i + 1)

            For j = 1 To i + 1
                multipleShifts(i).ShiftLeft()
            Next
        Next

        'Assert
        Dim upperBound = original.GetUpperBound(0)

        For i = 0 To testCount - 1
            For j = 0 To upperBound
                Assert.AreEqual(shiftsByMultiple(i)(j), multipleShifts(i)(j))
            Next
        Next
    End Sub

    <TestMethod>
    Public Sub ShiftLeftByMultipleMatchesMultipleShiftLeftsOnValueArray()
        'Arrange
        Dim original = {1, 2, 3, 4, 5}
        Dim testCount = 3 * original.Length
        Dim shiftsByMultiple = Enumerable.Range(1, testCount).Select(Function(n) original.ToArray()).ToArray()
        Dim multipleShifts = Enumerable.Range(1, testCount).Select(Function(n) original.ToArray()).ToArray()

        'Act
        For i = 0 To testCount - 1
            shiftsByMultiple(i).ShiftLeft(i + 1)

            For j = 1 To i + 1
                multipleShifts(i).ShiftLeft()
            Next
        Next

        'Assert
        Dim upperBound = original.GetUpperBound(0)

        For i = 0 To testCount - 1
            For j = 0 To upperBound
                Assert.AreEqual(shiftsByMultiple(i)(j), multipleShifts(i)(j))
            Next
        Next
    End Sub

#End Region 'ShiftLeft

#Region "ShiftRight"

    <TestMethod>
    Public Sub ShiftRightDoesNothingOnEmptyReferenceArray()
        'Arrange
        Dim source As String() = {}

        'Act
        source.ShiftRight()

        'Assert
        Assert.AreEqual(source.Length, 0)
    End Sub

    <TestMethod>
    Public Sub ShiftRightDoesNothingOnEmptyValueArray()
        'Arrange
        Dim source As Integer() = {}

        'Act
        source.ShiftRight()

        'Assert
        Assert.AreEqual(source.Length, 0)
    End Sub

    <TestMethod>
    Public Sub ShiftRightDoesNothingOnReferenceArrayWithOneElement()
        'Arrange
        Dim element = "Hello World"
        Dim source As String() = {element}

        'Act
        source.ShiftRight()

        'Assert
        Assert.AreEqual(source(0), element)
    End Sub

    <TestMethod>
    Public Sub ShiftRightDoesNothingOnValueArrayWithOneElement()
        'Arrange
        Dim element = 123
        Dim source As Integer() = {element}

        'Act
        source.ShiftRight()

        'Assert
        Assert.AreEqual(source(0), element)
    End Sub

    <TestMethod>
    Public Sub ShiftRightShiftsByOneOnReferenceArray()
        'Arrange
        Dim original = {"One", "Two", "Three", "Four", "Five"}
        Dim source = original.ToArray()

        'Act
        source.ShiftRight()

        'Assert
        Dim length = source.Length

        For i = 0 To source.GetUpperBound(0)
            Assert.AreEqual(source((i + 1) Mod length), original(i))
        Next
    End Sub

    <TestMethod>
    Public Sub ShiftRightShiftsByOneOnValueArray()
        'Arrange
        Dim original = {1, 2, 3, 4, 5}
        Dim source = original.ToArray()

        'Act
        source.ShiftRight()

        'Assert
        Dim length = source.Length

        For i = 0 To source.GetUpperBound(0)
            Assert.AreEqual(source((i + 1) Mod length), original(i))
        Next
    End Sub

    <TestMethod>
    Public Sub ShiftRightShiftsByCountLessThanLengthOnReferenceArray()
        'Arrange
        Dim original = {"One", "Two", "Three", "Four", "Five"}
        Dim source = original.ToArray()
        Dim count = 3

        'Act
        source.ShiftRight(count)

        'Assert
        Dim length = source.Length

        For i = 0 To source.GetUpperBound(0)
            Assert.AreEqual(source((i + count) Mod length), original(i))
        Next
    End Sub

    <TestMethod>
    Public Sub ShiftRightShiftsByCountLessThanLengthOnValueArray()
        'Arrange
        Dim original = {1, 2, 3, 4, 5}
        Dim source = original.ToArray()
        Dim count = 3

        'Act
        source.ShiftRight(count)

        'Assert
        Dim length = source.Length

        For i = 0 To source.GetUpperBound(0)
            Assert.AreEqual(source((i + count) Mod length), original(i))
        Next
    End Sub

    <TestMethod>
    Public Sub ShiftRightShiftsByCountEqualToThanLengthOnReferenceArray()
        'Arrange
        Dim original = {"One", "Two", "Three", "Four", "Five"}
        Dim source = original.ToArray()
        Dim length = source.Length

        'Act
        source.ShiftRight(length)

        'Assert

        For i = 0 To source.GetUpperBound(0)
            Assert.AreEqual(source((i + length) Mod length), original(i))
        Next
    End Sub

    <TestMethod>
    Public Sub ShiftRightShiftsByCountEqualToLengthOnValueArray()
        'Arrange
        Dim original = {1, 2, 3, 4, 5}
        Dim source = original.ToArray()
        Dim length = source.Length

        'Act
        source.ShiftRight(length)

        'Assert

        For i = 0 To source.GetUpperBound(0)
            Assert.AreEqual(source((i + length) Mod length), original(i))
        Next
    End Sub

    <TestMethod>
    Public Sub ShiftRightShiftsByCountGreaterThanLengthOnReferenceArray()
        'Arrange
        Dim original = {"One", "Two", "Three", "Four", "Five"}
        Dim source = original.ToArray()
        Dim count = 13

        'Act
        source.ShiftRight(count)

        'Assert
        Dim length = source.Length

        For i = 0 To source.GetUpperBound(0)
            Assert.AreEqual(source((i + count) Mod length), original(i))
        Next
    End Sub

    <TestMethod>
    Public Sub ShiftRightShiftsByCountGreaterThanLengthOnValueArray()
        'Arrange
        Dim original = {1, 2, 3, 4, 5}
        Dim source = original.ToArray()
        Dim count = 13

        'Act
        source.ShiftRight(count)

        'Assert
        Dim length = source.Length

        For i = 0 To source.GetUpperBound(0)
            Assert.AreEqual(source((i + count) Mod length), original(i))
        Next
    End Sub

    <TestMethod>
    Public Sub ShiftRightByMultipleMatchesMultipleShiftRightsOnReferenceArray()
        'Arrange
        Dim original = {"One", "Two", "Three", "Four", "Five"}
        Dim testCount = 3 * original.Length
        Dim shiftsByMultiple = Enumerable.Range(1, testCount).Select(Function(n) original.ToArray()).ToArray()
        Dim multipleShifts = Enumerable.Range(1, testCount).Select(Function(n) original.ToArray()).ToArray()

        'Act
        For i = 0 To testCount - 1
            shiftsByMultiple(i).ShiftRight(i + 1)

            For j = 1 To i + 1
                multipleShifts(i).ShiftRight()
            Next
        Next

        'Assert
        Dim upperBound = original.GetUpperBound(0)

        For i = 0 To testCount - 1
            For j = 0 To upperBound
                Assert.AreEqual(shiftsByMultiple(i)(j), multipleShifts(i)(j))
            Next
        Next
    End Sub

    <TestMethod>
    Public Sub ShiftRightByMultipleMatchesMultipleShiftRightsOnValueArray()
        'Arrange
        Dim original = {1, 2, 3, 4, 5}
        Dim testCount = 3 * original.Length
        Dim shiftsByMultiple = Enumerable.Range(1, testCount).Select(Function(n) original.ToArray()).ToArray()
        Dim multipleShifts = Enumerable.Range(1, testCount).Select(Function(n) original.ToArray()).ToArray()

        'Act
        For i = 0 To testCount - 1
            shiftsByMultiple(i).ShiftRight(i + 1)

            For j = 1 To i + 1
                multipleShifts(i).ShiftRight()
            Next
        Next

        'Assert
        Dim upperBound = original.GetUpperBound(0)

        For i = 0 To testCount - 1
            For j = 0 To upperBound
                Assert.AreEqual(shiftsByMultiple(i)(j), multipleShifts(i)(j))
            Next
        Next
    End Sub

#End Region 'ShiftRight

#Region "GetNext"

    <TestMethod>
    Public Sub GetNextThrowsExceptionOnEmptyReferenceArray()
        'Arrange
        Dim source As String() = {}
        Dim index = 0
        Dim exception As Exception = Nothing

        'Act
        Try
            source.GetNext(index)
        Catch ex As Exception
            exception = ex
        End Try

        'Assert
        Assert.IsInstanceOfType(exception, GetType(IndexOutOfRangeException))
    End Sub

    <TestMethod>
    Public Sub GetNextThrowsExceptionOnEmptyValueArray()
        'Arrange
        Dim source As Integer() = {}
        Dim index = 0
        Dim exception As Exception = Nothing

        'Act
        Try
            source.GetNext(index)
        Catch ex As Exception
            exception = ex
        End Try

        'Assert
        Assert.IsInstanceOfType(exception, GetType(IndexOutOfRangeException))
    End Sub

    <TestMethod>
    Public Sub GetNextSucceedsWithoutWrappingOnReferenceArray()
        'Arrange
        Dim source = {"One", "Two", "Three", "Four", "Five"}
        Dim originalIndex = 0
        Dim finalIndex = originalIndex

        'Act
        Dim result = source.GetNext(finalIndex)

        'Assert
        Assert.AreEqual(finalIndex, originalIndex + 1)
        Assert.AreEqual(result, source(finalIndex))
    End Sub

    <TestMethod>
    Public Sub GetNextSucceedsWithoutWrappingOnValueArray()
        'Arrange
        Dim source = {1, 2, 3, 4, 5}
        Dim originalIndex = 0
        Dim finalIndex = originalIndex

        'Act
        Dim result = source.GetNext(finalIndex)

        'Assert
        Assert.AreEqual(finalIndex, originalIndex + 1)
        Assert.AreEqual(result, source(finalIndex))
    End Sub

    <TestMethod>
    Public Sub GetNextSucceedsWithWrappingOnReferenceArray()
        'Arrange
        Dim source = {"One", "Two", "Three", "Four", "Five"}
        Dim originalIndex = source.GetUpperBound(0)
        Dim finalIndex = originalIndex

        'Act
        Dim result = source.GetNext(finalIndex)

        'Assert
        Assert.AreEqual(finalIndex, 0)
        Assert.AreEqual(result, source(finalIndex))
    End Sub

    <TestMethod>
    Public Sub GetNextSucceedsWithWrappingOnValueArray()
        'Arrange
        Dim source = {1, 2, 3, 4, 5}
        Dim originalIndex = source.GetUpperBound(0)
        Dim finalIndex = originalIndex

        'Act
        Dim result = source.GetNext(finalIndex)

        'Assert
        Assert.AreEqual(finalIndex, 0)
        Assert.AreEqual(result, source(finalIndex))
    End Sub

#End Region 'GetNext

#Region "GetPrevious"

    <TestMethod>
    Public Sub GetPreviousThrowsExceptionOnEmptyReferenceArray()
        'Arrange
        Dim source As String() = {}
        Dim index = 0
        Dim exception As Exception = Nothing

        'Act
        Try
            source.GetPrevious(index)
        Catch ex As Exception
            exception = ex
        End Try

        'Assert
        Assert.IsInstanceOfType(exception, GetType(IndexOutOfRangeException))
    End Sub

    <TestMethod>
    Public Sub GetPreviousThrowsExceptionOnEmptyValueArray()
        'Arrange
        Dim source As Integer() = {}
        Dim index = 0
        Dim exception As Exception = Nothing

        'Act
        Try
            source.GetPrevious(index)
        Catch ex As Exception
            exception = ex
        End Try

        'Assert
        Assert.IsInstanceOfType(exception, GetType(IndexOutOfRangeException))
    End Sub

    <TestMethod>
    Public Sub GetPreviousSucceedsWithoutWrappingOnReferenceArray()
        'Arrange
        Dim source = {"One", "Two", "Three", "Four", "Five"}
        Dim originalIndex = source.GetUpperBound(0)
        Dim finalIndex = originalIndex

        'Act
        Dim result = source.GetPrevious(finalIndex)

        'Assert
        Assert.AreEqual(finalIndex, originalIndex - 1)
        Assert.AreEqual(result, source(finalIndex))
    End Sub

    <TestMethod>
    Public Sub GetPreviousSucceedsWithoutWrappingOnValueArray()
        'Arrange
        Dim source = {1, 2, 3, 4, 5}
        Dim originalIndex = source.GetUpperBound(0)
        Dim finalIndex = originalIndex

        'Act
        Dim result = source.GetPrevious(finalIndex)

        'Assert
        Assert.AreEqual(finalIndex, originalIndex - 1)
        Assert.AreEqual(result, source(finalIndex))
    End Sub

    <TestMethod>
    Public Sub GetPreviousSucceedsWithWrappingOnReferenceArray()
        'Arrange
        Dim source = {"One", "Two", "Three", "Four", "Five"}
        Dim originalIndex = 0
        Dim finalIndex = originalIndex

        'Act
        Dim result = source.GetPrevious(finalIndex)

        'Assert
        Assert.AreEqual(finalIndex, source.GetUpperBound(0))
        Assert.AreEqual(result, source(finalIndex))
    End Sub

    <TestMethod>
    Public Sub GetPreviousSucceedsWithWrappingOnValueArray()
        'Arrange
        Dim source = {1, 2, 3, 4, 5}
        Dim originalIndex = 0
        Dim finalIndex = originalIndex

        'Act
        Dim result = source.GetPrevious(finalIndex)

        'Assert
        Assert.AreEqual(finalIndex, source.GetUpperBound(0))
        Assert.AreEqual(result, source(finalIndex))
    End Sub

#End Region 'GetPrevious

End Class