﻿<TestClass> Public Class EnumerableExtensionsTest

#Region "Randomize"

    <TestMethod>
    Public Sub RandomizeThrowsOnNullReference()
        'Arrange
        Dim source As IEnumerable(Of Object)
        Dim exception As Exception

        'Act
        Try
            source.Randomize()
        Catch ex As Exception
            exception = ex
        End Try

        'Assert
        Assert.IsNotNull(exception)
    End Sub

    <TestMethod>
    Public Sub RandomizeReturnsSameItems()
        'Arrange
        Dim source = Enumerable.Range(1, 10).
                                Select(Function(n) New Object).
                                ToArray()

        'Act
        Dim result = source.Randomize().ToArray()

        'Assert
        Assert.AreEqual(source.Length, result.Length)
        Assert.IsTrue(source.All(Function(o) result.Contains(o)))
    End Sub

    <TestMethod>
    Public Sub RandomizeChangesOrder()
        'Arrange
        Dim source = Enumerable.Range(1, 10).
                                Select(Function(n) New Object).
                                ToArray()

        'Act
        Dim result = source.Randomize().ToArray()

        'Assert
        Assert.IsTrue(Enumerable.Range(0, source.Length).Any(Function(n) source(n) IsNot result(n)))
    End Sub

#End Region 'Randomize

#Region "Randomize(Random)"

    <TestMethod>
    Public Sub RandomizeWithRandomThrowsOnNullReference()
        'Arrange
        Dim source As IEnumerable(Of Object)
        Dim random As New Random()
        Dim exception As Exception

        'Act
        Try
            source.Randomize(random)
        Catch ex As Exception
            exception = ex
        End Try

        'Assert
        Assert.IsNotNull(exception)
    End Sub

    <TestMethod>
    Public Sub RandomizeWithRandomReturnsSameItems()
        'Arrange
        Dim source = Enumerable.Range(1, 10).
                                Select(Function(n) New Object).
                                ToArray()
        Dim random = New Random()

        'Act
        Dim result = source.Randomize(random).ToArray()

        'Assert
        Assert.AreEqual(source.Length, result.Length)
        Assert.IsTrue(source.All(Function(o) result.Contains(o)))
    End Sub

    <TestMethod>
    Public Sub RandomizeWithRandomChangesOrder()
        'Arrange
        Dim source = Enumerable.Range(1, 10).
                                Select(Function(n) New Object).
                                ToArray()
        Dim random = New Random()

        'Act
        Dim result = source.Randomize(random).ToArray()

        'Assert
        Assert.IsTrue(Enumerable.Range(0, source.Length).Any(Function(n) source(n) IsNot result(n)))
    End Sub

    <TestMethod>
    Public Sub RandomizeWithRandomProducesExpectedOrder()
        'Arrange
        Dim source = Enumerable.Range(1, 10).ToArray()
        Dim seed = Convert.ToInt32(DateTime.Now.Ticks Mod Integer.MaxValue)
        Dim random1 = New Random(seed)
        Dim random2 = New Random(seed)

        'Act
        Dim result = source.Randomize(random1).ToArray()
        Dim keys = Enumerable.Range(1, source.Length).
                              Select(Function(n) random2.NextDouble()).
                              ToArray()

        Array.Sort(keys, source)

        'Assert
        Assert.IsTrue(Enumerable.Range(0, source.Length).All(Function(n) source(n) = result(n)))
    End Sub

#End Region 'Randomize(Random)

End Class